"""Qbus MQTT API constants."""

TOPIC_PREFIX = "cloudapp/QBUSMQTTGW"

KEY_OUTPUT_ACTION = "action"
KEY_OUTPUT_ACTIONS = "actions"
KEY_OUTPUT_ID = "id"
KEY_OUTPUT_LOCATION = "location"
KEY_OUTPUT_LOCATION_ID = "locationId"
KEY_OUTPUT_NAME = "name"
KEY_OUTPUT_PROPERTIES = "properties"
KEY_OUTPUT_REF_ID = "refId"
KEY_OUTPUT_TYPE = "type"

KEY_PROPERTIES_AUTHKEY = "authKey"
KEY_PROPERTIES_SHUTTER_POSITION = "shutterPosition"
KEY_PROPERTIES_SLAT_POSITION = "slatPosition"
KEY_PROPERTIES_STATE = "state"
KEY_PROPERTIES_VALUE = "value"
